function DC_OL =findDC_OL_Sim(ntf,n,aantal_ptn, averaging, seed)

%  Finds the prediction of the DC overloading level trhough time domain
%  simulation (requires Schreier's Delta Sigma Toolbox)
%
%  ntf : the noise transfer function
%  n   : the number of unit elements 
%        must be even in the current version
%  aantal_ptn  :  nr of points in time domain simulations
%  averaging   :  nr of times to average the time domain simulation of
%                 aantal_ptn data points
% seed :  to initialize the random number generator (to obtain reproducible
%         results
%
% (C) Pieter Rombouts
      
      


% Handle the input arguments
% Code Fragment inspired on code from
% Richard Schreier's delta sigma Toolbox
parameters = {'ntf' 'n' 'aantal_ptn' 'averaging' 'seed' };
defaults = {[] 8 2^16 4 1 };
for arg_i=2:length(defaults)
    parameter = char(parameters(arg_i));
    if arg_i>nargin | ( eval(['isnumeric(' parameter ') '])  &  ...
     eval(['any(isnan(' parameter ')) | isempty(' parameter ') ']) )
        eval([parameter '=defaults{arg_i};'])
    end
end
% End of handling of the input arguments

accuracy = 0.003;
decision_accuracy = 0.03; % not too small to avoid that ntf's without zeros at DC (e.g. 4th order) would crash
offset = 100;

order  = length(ntf.P{1});
rand('state',seed);

nlev = n+1;

DC_low = max(0.01, findOL_Carley(ntf,n));
DC_high = 1+0.1/pi; % weird upper limiet to avoid getting locked in anomalous DC states
stopcriterium = 0;
while ~stopcriterium
  DC_candidate = 0.5*(DC_low + DC_high);
  stabiel=1; 
  for j=1:averaging,
      u = DC_candidate*ones(1,aantal_ptn+offset)*n; 
      x0 = (2*rand(order,1)-1)*0.125; % start with a dithered state to reduce pattern noise
      [v,xn,xmax,y] = simulateDSM(u,ntf,nlev,x0);
      D_out_exp = (mean(v(offset+1:end))/(nlev-1));
			  % don't use startup points that are affected by the initial state
      quant_in_max= max(abs(y(offset+1:end)/(nlev-1)));
			  % don't use startup points that are affected by the initial state
      if abs(D_out_exp-DC_candidate)>decision_accuracy, stabiel =0; continue; end %if
      if quant_in_max>10, stabiel =0; continue; end %if
  end % for j=1:averaging,
  if stabiel, DC_low = DC_candidate; else DC_high= DC_candidate; end %if
  if (DC_high-DC_low)<accuracy, stopcriterium = 1; end %if
end % for
DC_OL = DC_low;