function [Kn,Ks,sigma_Qsat,converge,stabiel, sigma_Xn] =findDescFunctDC(ntf,C,n,KnStart,KsStart, mode)

%  Finds the describing functions for a modulator 
% (corresponds to the algorithm of Fig. 8)
%
%  ntf : the noise transfer function
%  C   : DC input level
%  n   : the number of unit elements 
%        must be even in the current version
%  KnStart   : optional start value of the iterative algorithm
%  KsStart   : optional start value of the iterative algorithm
%  mode = 0;  assume that the quantisation noise is white (default)
%         1;  assume that the quantisation noise has a worst-case spectrum
%
% (C) Pieter Rombouts
      

% Handle the input arguments
% Code Fragment inspired on code from
% Richard Schreier's delta sigma Toolbox
parameters = {'ntf' 'C' 'n' 'KnStart' 'KsStart' 'mode'};
defaults = {[] 0.5 8 1 1 0};
for arg_i=2:length(defaults)
    parameter = char(parameters(arg_i));
    if arg_i>nargin | ( eval(['isnumeric(' parameter ') '])  &  ...
     eval(['any(isnan(' parameter ')) | isempty(' parameter ') ']) )
        eval([parameter '=defaults{arg_i};'])
    end
end
% End of handling of the input arguments


omegas = 0:0.01:pi;
ntf_omega = squeeze(freqresp(ntf,omegas));
H_omega = 1./ntf_omega  -1; H_omega(find((abs(ntf_omega)==0)))=10e3;
	%  figure
	%  plot(abs(H_omega))

	%  figure
	%  plot(abs(ntf_omega)); hold on
	%  plot(abs(1./(1+H_omega)),'xr');
[Z,P,K] = zpkdata(ntf);
H_num = poly(P{1})-K*poly(Z{1});
H_den = K*poly(Z{1});

accuracy = 0.0001;
max_iterations = 50;

Kn = KnStart;
Ks = KsStart;
sigma_Qsat = 2/n/sqrt(12);

stopcriterium = 0;
NrIterations = 0;
converge = 1;
stabiel = 1;
 
while ~stopcriterium,
  X_DC = C/Ks;
  ntf_den = Kn*H_num +H_den; ntf_poles = roots(ntf_den);
  stabiel  = isempty(find(abs(ntf_poles)>1));
  if ~stabiel,
%     stopcriterium = 1; Kn=NaN; Ks=NaN; sigma_Qsat=NaN; converge=NaN; sigma_Xn=NaN; return;
      stopcriterium = 1; Kn=0; Ks=0; sigma_Qsat=Inf; converge=1; sigma_Xn=Inf; return;
  end %if
  Qsat_noise_Gain  = H_omega./(1+Kn*H_omega);
	      %    Qsat_noise_Gain(2)
	      %    figure
	      %    plot((abs(Qsat_noise_Gain)));
  if mode==0, 
      RMS_Qsat_noise_Gain = sqrt(mean(abs(Qsat_noise_Gain).^2));
      sigma_Xn = sigma_Qsat*RMS_Qsat_noise_Gain;
  else % mode == 1
      Max_noise_Gain = max(abs(Qsat_noise_Gain));
      sigma_Xn = sigma_Qsat*Max_noise_Gain;
  end % if mode
  Kn_new=KnDC(X_DC,sigma_Xn,n);
  Ks_new=KsDC(X_DC,sigma_Xn,n);
  sigma_Qsat_new = sigmaQsatDC(X_DC,sigma_Xn,Kn_new,X_DC*Ks_new,n);
  if ((abs(sigma_Qsat_new - sigma_Qsat))^2 + (abs(Kn_new - Kn))^2 + (abs(Ks_new - Ks))^2+ (abs(Ks_new*X_DC - C))^2) < accuracy^2, stopcriterium = 1; end %if
  Kn = Kn_new;
  Ks = Ks_new;
  sigma_Qsat =   sigma_Qsat_new;
  NrIterations = NrIterations+1;
  if NrIterations>max_iterations, converge = 0; stopcriterium = 1; end %if
end % while ~stopcriterium