function Sin_OL =findSin_OL_Sim_ABCD(ntf,n,f_0, aantal_ptn, averaging, seed)

%  Finds the prediction of the sinusoidal overloading level 
%  trhough time domain simulation 
%  (requires Schreier's Delta Sigma Toolbox)
%
%  ntf : the noise transfer function
%  n   : the number of unit elements 
%        must be even in the current version
%  f_0   : test frequency
%  aantal_ptn  :  nr of points in time domain simulations
%  averaging   :  nr of times to average the time domain simulation of
%                 aantal_ptn data points
% seed :  to initialize the random number generator (to obtain reproducible
%         results
%
% (C) Pieter Rombouts


% Handle the input arguments
% Code Fragment inspired on code from
% Richard Schreier's delta sigma Toolbox
parameters = {'ntf' 'n' 'f_0' 'aantal_ptn' 'averaging' 'seed' };
defaults = {[] 8 0.5/32/7.34656 2^16 4 1 };
for arg_i=2:length(defaults)
    parameter = char(parameters(arg_i));
    if arg_i>nargin | ( eval(['isnumeric(' parameter ') '])  &  ...
     eval(['any(isnan(' parameter ')) | isempty(' parameter ') ']) )
        eval([parameter '=defaults{arg_i};'])
    end
end
% End of handling of the input arguments

accuracy = 0.003;
amp_accuracy = 0.1;
offset = 100;

order  = length(ntf.P{1});
rand('state',seed);

nlev = n+1;

Sin_low = max(0.01, findOL_Carley(ntf,n));
Sin_high = 1+1/n+0.1/pi; % weird upper limiet to avoid getting locked in anomalous Sin states

% Sin_low = 0.63;
% Sin_high =0.63;
stopcriterium = 0;
while ~stopcriterium
  Sin_candidate = 0.5*(Sin_low + Sin_high)
  stabiel=1; 
  for j=1:averaging,
      j
      u = Sin_candidate*sin(2*pi*f_0*(1:aantal_ptn+offset))*n; 
      x0 = (2*rand(order,1)-1)*0.125; % start with a dithered state to reduce pattern noise
      
      % original version
%      [v,xn,xmax,y] = simulateDSM(u,ntf,nlev,x0);

              
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	        % calculate the ABCD-matrix 
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	        order = length(ntf.z{:});
	        [A,B2,C,D2] = zp2ss(ntf.p{:},ntf.z{:},-1);	% A realization of 1/H
	        % Transform the realization so that C = [1 0 0 ...]
	        Sinv = orth([C' eye(order)])/norm(C); S = inv(Sinv);
	        C = C*Sinv;
	        if C(1)<0
	    	S = -S;
	    	Sinv = -Sinv;
	        end
	        A = S*A*Sinv; B2 = S*B2; C = [1 zeros(1,order-1)]; % C=C*Sinv; 
	        D2 = 0;
	        % !!!! Assume stf=1
	        %B1 = -B2;
	        B1 = [-B2(1,1) ; zeros(order-1,1)];
	        % D1 = 1;
	        D1 = 0;
	        B = [B1 B2];
	        ABCD = zeros(order+1, order+2);
	        ABCD(1:order, 1:order) = A;
	        ABCD(1:order, order+1:order+2)= B;
	        ABCD(order+1, 1:order) = C;
	        ABCD(order+1, order+1) = D1;
	        [v,xn,xmax,y] = simulateDSM(u,ABCD,nlev,x0);
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	        % end of calculation of the ABCD-matrix 
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      quant_in_max= max(abs(y(1:end)))/(nlev-1)
%      [SNR, A_D,B_D,C_D, errorD] = Sinfit_fixed_freq(v(offset+1:end),f_0);
			  % don't use startup points that are affected by the initial state
%      if abs(sqrt(A_D^2+B_D^2)/n-Sin_candidate)>amp_accuracy, stabiel =0; disp('amp_error'); continue; end %if
%      if abs(sqrt(A_D^2+B_D^2)/n-Sin_candidate)>1, stabiel =0; disp('amp_error'); continue; end %if
%      abs(sqrt(A_D^2+B_D^2)/n)
      if quant_in_max>10, 
          stabiel =0;
          break; 
      end %if
  end % for j=1:averaging,
  if stabiel, 
     Sin_low = Sin_candidate;
  else Sin_high= Sin_candidate; 
  end %if
  if (Sin_high-Sin_low)<accuracy, stopcriterium = 1; end %if
end % for
Sin_OL = Sin_low;