function sigma=sigmaQsatDC(X_DC,sigma_Xn,Kn,C,n)

%  Finds the best rms value of the linearization error for a quantizer.
% (corresponds to eq. (14))
%
%  X_DC       : the dc level at the quantizer input
%  sigma_Xn   : the noise rms value at  the quantizer input
%  Kn : best fit gain Kn for a quantizer (must be determined first)
%  C  : the output level of the quantizer
%  n   : the number of unit elements 
%        must be even in the current version
%
% (C) Pieter Rombouts

% Handle the input arguments
% Code Fragment inspired on code from
% Richard Schreier's delta sigma Toolbox
parameters = {'X_DC' 'sigma_Xn' 'Kn' 'C' 'n'};
defaults = {0.5 0.1 [] [] 8};
for arg_i=2:length(defaults)
    parameter = char(parameters(arg_i));
    if arg_i>nargin | ( eval(['isnumeric(' parameter ') '])  &  ...
     eval(['any(isnan(' parameter ')) | isempty(' parameter ') ']) )
        eval([parameter '=defaults{arg_i};'])
    end
end
% End of handling of the input arguments



delta = 2/n;
i=0:n/2-1;

erf1_arguments = (-X_DC +delta/2+i*delta)/(sqrt(2)*sigma_Xn);
erf2_arguments = (-X_DC -delta/2-i*delta)/(sqrt(2)*sigma_Xn);
contributions = delta^2*(2*i+1)/2.*(2-erf(erf1_arguments)+erf(erf2_arguments));
MS_Qsat = sum(contributions)-Kn^2*sigma_Xn^2-C^2;
if MS_Qsat<0, MS_Qsat = 0; end%if
sigma = sqrt(MS_Qsat);
