function sigma=sigmaQsatSin(a,sigma,Kn,Ks,n)

%  Finds the best rms value of the linearization error for a quantizer.
% (corresponds to eq. (21))
%
%  a       : the amplitude at the quantizer input
%  sigma   : the noise rms value at  the quantizer input
%  Kn : best fit gain Kn for a quantizer (must be determined first)
%  Kn : best fit gain Ks for a quantizer (must be determined first)
%  n   : the number of unit elements 
%        must be even in the current version
%
% (C) Pieter Rombouts


% Handle the input arguments
% Code Fragment inspired on code from
% Richard Schreier's delta sigma Toolbox
parameters = {'X_DC' 'sigma' 'Kn' 'Ks' 'n'};
defaults = {0.5 0.1 [] [] 8};
for arg_i=2:length(defaults)
    parameter = char(parameters(arg_i));
    if arg_i>nargin | ( eval(['isnumeric(' parameter ') '])  &  ...
     eval(['any(isnan(' parameter ')) | isempty(' parameter ') ']) )
        eval([parameter '=defaults{arg_i};'])
    end
end
% End of handling of the input arguments



step = 0.01;
thetarange = (-1:step:1)*pi/2;
    

length(thetarange);

MS_Qsat = 0;

delta = 2/n;
for i=0:n/2-1,
    B = -delta/2-i*delta;
    integs_args = erf((a*sin(thetarange)+B)/sigma/sqrt(2))+1;
    integ = trapz(thetarange,integs_args)*(2*i+1);
    MS_Qsat = MS_Qsat + integ;
    integs_args = erf((-a*sin(thetarange)+B)/sigma/sqrt(2))+1;
    integ = trapz(thetarange,integs_args)*(2*i+1);
    MS_Qsat = MS_Qsat + integ;
end % for i
MS_Qsat  = MS_Qsat * delta^2/2/pi;

MS_Qsat = MS_Qsat-Kn^2*sigma^2-(a*Ks)^2/2;
if MS_Qsat<0, MS_Qsat = 0; end%if
sigma = sqrt(MS_Qsat);

